"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import os
from typing import Union

from Setter.Common.GatewayImage import GatewayImage


class BootFlagManager:
	"""
	Boot flag preventing repeated configuration manager.

	Methods
	-------
	create_flag() -> None:
		Create flag preventing repeated configuration.
	check_flag() -> bool:
		Check if flag preventing repeated configuration is present.
	"""

	_path: str = '/var/cache/iqrf-gateway-setter'

	def __init__(self, image: Union[GatewayImage, None]):
		"""
		Construct manager.

		Parameters
		----------
		image: Union[GatewayImage, None]
			Gateway image information
		"""
		if image is not None and image.os_type == 'yocto':
			self._path = '/data/setter/bootpart'

	@classmethod
	def create_flag(cls) -> None:
		"""
		Create flag preventing repeated configuration.

		Presence of flag file is checked before gateway configuration is executed.
		Gateway is configured if flag file does not exist or if utility is invoked with --force options.

		Returns
		-------
		None
		"""
		directory = os.path.dirname(cls._path)
		if not os.path.exists(directory):
			os.makedirs(directory, exist_ok=True)
		with open(cls._path, mode='a', encoding='utf-8'):
			os.utime(cls._path, None)

	@classmethod
	def check_flag(cls) -> bool:
		"""
		Check if flag preventing repeated configuration is present.

		Presence of flag file is checked before gateway configuration is executed.
		Gateway is configured if flag file does not exist or if utility is invoked with --force options.

		Returns
		-------
		bool
			True if flag file is present, False otherwise
		"""
		return os.path.exists(cls._path)
