"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


class SpiConfig:
	"""
	SPI interface configuration class.

	Attributes
	----------
	device: str
		device path
	power_pin: int
		power pin
	pgm_pin: int
		programming mode pin
	bus_pin: int
		bus enable pin
	spi_reset: bool
		reset spi on boot
	i2c_pin: int
		i2c interface pin
	spi_pin: int
		spi interface pin
	uart_pin: int
		uart interface pin

	Methods
	-------
	to_daemon() -> dict:
		Serialize SPI configuration into daemon compliant dictionary.
	to_uploader() -> dict:
		Serialize SPI configuration into uploader compliant dictionary.
	"""

	def __init__(self, device: str, power_pin: int, pgm_pin: int, bus_pin: int,
				spi_reset: bool, i2c_pin=None, spi_pin=None, uart_pin=None) -> None:
		"""
		Construct attributes for spi configuration object.

		Parameters
		----------
		device: str
			device path
		power_pin: int
			power pin
		pgm_pin: int
			programming mode pin
		bus_pin: int
			bus enable pin
		spi_reset: bool
			reset spi on boot
		i2c_pin: int
			i2c interface pin
		spi_pin: int
			spi interface pin
		uart_pin: int
			uart interface pin
		"""
		self.device: str = device
		self.power_pin: int = power_pin
		self.pgm_pin: int = pgm_pin
		self.bus_pin: int = bus_pin
		self.spi_reset: bool = spi_reset
		self.i2c_pin: int = i2c_pin
		self.spi_pin: int = spi_pin
		self.uart_pin: int = uart_pin

	def to_daemon(self) -> dict:
		"""
		Serialize SPI configuration into daemon compliant dictionary.

		Parameters
		----------
		None

		Returns
		-------
		dict
			SPI configuration serialized into dictionary.
		"""
		config = {
			'component': 'iqrf::IqrfSpi',
			'instance': 'iqrf::IqrfSpi-' + self.device,
			'IqrfInterface': self.device,
			'powerEnableGpioPin': self.power_pin,
			'busEnableGpioPin': self.bus_pin,
			'pgmSwitchGpioPin': self.pgm_pin,
			'spiReset': self.spi_reset
		}
		if self.i2c_pin is not None:
			config['i2cEnableGpioPin'] = self.i2c_pin
		if self.spi_pin is not None:
			config['spiEnableGpioPin'] = self.spi_pin
		if self.uart_pin is not None:
			config['uartEnableGpioPin'] = self.uart_pin
		return config

	def to_uploader(self) -> dict:
		"""
		Serialize SPI configuration into uploader compliant dictionary.

		Parameters
		----------
		None

		Returns
		-------
		dict
			SPI configuration serialized into dictionary.
		"""
		config = {
			'spiDev': self.device,
			'powerPin': self.power_pin,
			'pgmPin': self.pgm_pin,
			'busPin': self.bus_pin,
			'spiReset': self.spi_reset
		}
		if self.i2c_pin is not None:
			config['i2cPin'] = self.i2c_pin
		if self.spi_pin is not None:
			config['spiPin'] = self.spi_pin
		if self.uart_pin is not None:
			config['uartPin'] = self.uart_pin
		return config
