"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import os
from logging import Logger
from typing import List

from Setter.Common.VersionParser import VersionParser
from Setter.Common.FileManager import FileManager
from Setter.Common.ServiceManager import ServiceManager
from Setter.Enums.Software import SoftwareCommand, SoftwareBackup, FactoryResetSoftware, SystemdServices
from Setter.Exceptions import VersionParseException
from Setter.Exceptions.CommandException import CommandException
from Setter.Exceptions.IOErrorException import IOErrorException
from Setter.Exceptions.ServiceErrorException import ServiceErrorException
from Setter.Logging import Logging
from Setter.WebappConfig import WebappConfig


class FactoryReset:
	"""Factory reset class."""

	_backup_dir: str = '/usr/share/iqrf-gateway/default'
	_logger: Logger = Logging.get_logger()

	@classmethod
	def perform_reset(cls, software: List[FactoryResetSoftware], reset_all: bool) -> None:
		"""
		Perform factory reset for selected software of for all.

		Parameters
		----------
		software: List[FactoryResetSoftware]
			Software configuration to factory reset
		reset_all: bool
			Reset all software managed by backup

		Returns
		-------
		None
		"""
		services_to_reset = []
		# reset gateway file
		FactoryReset.restore_gateway_file()
		# reset cloud provisioning
		if FactoryResetSoftware.CLOUD_PROVISIONING in software or reset_all:
			FactoryReset.reset_cloud_provisioning()
			services_to_reset.append(SystemdServices.CLOUD_PROVISIONING)
		# reset daemon
		if FactoryResetSoftware.DAEMON in software or reset_all:
			FactoryReset.reset_daemon()
			services_to_reset.append(SystemdServices.DAEMON)
		# reset iqaros
		if FactoryResetSoftware.IQAROS in software or reset_all:
			FactoryReset.reset_iqaros()
		# reset network manager
		if FactoryResetSoftware.NETWORK_MANAGER in software or reset_all:
			FactoryReset.reset_network_manager()
			services_to_reset.append(SystemdServices.NETWORK_MANAGER)
		# reset webapp
		if FactoryResetSoftware.WEBAPP in software or reset_all:
			FactoryReset.reset_webapp()
		# restart services
		for service in services_to_reset:
			if ServiceManager.exists(service):
				try:
					ServiceManager.restart(service)
				except ServiceErrorException as service_error:
					cls._logger.error('Failed to restart %s service. Error: %s', service, service_error.message)

	@classmethod
	def restore_gateway_file(cls):
		"""
		Perform gateway file factory reset.

		Returns
		-------
		None
		"""
		cls._logger.info('Performing gateway file factory reset.')
		try:
			if os.path.exists(f'{cls._backup_dir}/iqrf-gateway.json'):
				FileManager.copy(
					source=f'{cls._backup_dir}/iqrf-gateway.json',
					destination='/etc/iqrf-gateway.json',
				)
				cls._logger.info('Gateway file factory reset successful.')
		except IOErrorException as io_error:
			cls._logger.error(str(io_error))
			raise io_error

	@classmethod
	def reset_cloud_provisioning(cls):
		"""
		Perform Cloud Provisioning factory reset.

		Returns
		-------
		None
		"""
		cls._logger.info('Performing IQRF Cloud Provisioning factory reset.')
		try:
			FileManager.copy_tree(
				source=f'{cls._backup_dir}/{SoftwareBackup.CLOUD_PROVISIONING}/config/',
				destination='/etc/iqrf-cloud-provisioning',
			)
			FileManager.copy_tree(
				source=f'{cls._backup_dir}/{SoftwareBackup.CLOUD_PROVISIONING}/certs/',
				destination='/usr/share/iqrf-cloud-provisioning/cert',
			)
			cls._logger.info('IQRF Cloud Provisioning factory reset successful.')
		except IOErrorException as io_error:
			cls._logger.error(str(io_error))
			raise io_error

	@classmethod
	def reset_daemon(cls):
		"""
		Perform Daemon factory reset.

		Returns
		-------
		None
		"""
		cls._logger.info('Performing IQRF Gateway Daemon factory reset.')
		try:
			daemon_version = VersionParser.get_software_version(sw=SoftwareCommand.DAEMON)
			FileManager.copy_tree(
				source=f'{cls._backup_dir}/{SoftwareBackup.DAEMON}/config/',
				destination='/etc/iqrf-gateway-daemon',
			)
			FileManager.copy_tree(
				source=f'{cls._backup_dir}/{SoftwareBackup.DAEMON}/cfgSchemas/',
				destination='/usr/share/iqrf-gateway-daemon/cfgSchemas',
			)
			FileManager.copy_tree(
				source=f'{cls._backup_dir}/{SoftwareBackup.DAEMON}/schedulerSchemas/',
				destination='/usr/share/iqrf-gateway-daemon/schedulerSchemas',
			)
			FileManager.copy_tree(
				source=f'{cls._backup_dir}/{SoftwareBackup.DAEMON}/scheduler/',
				destination='/var/cache/iqrf-gateway-daemon/scheduler',
			)
			if daemon_version.major == 2 and daemon_version.minor >= 6 or daemon_version.major == 3:
				FileManager.copy_tree(
					source=f'{cls._backup_dir}/{SoftwareBackup.DAEMON}/cacheSchemas/',
					destination='/usr/share/iqrf-gateway-daemon/cacheSchemas'
				)
			cls._logger.info('IQRF Gateway Daemon factory reset successful.')
		except (CommandException, IOErrorException, VersionParseException) as error:
			cls._logger.error(str(error))
			raise error

	@classmethod
	def reset_iqaros(cls):
		"""
		Perform IQAROS factory reset.

		Returns
		-------
		None
		"""
		cls._logger.info('Performing IQAROS factory reset.')
		result = WebappConfig.run_manager_command('user:remove-all --role basic --no-interaction')
		if result.returncode != 0:
			error_str = f'Failed to remove IQAROS users: {result.stderr}.'
			cls._logger.error(error_str)
			raise CommandException(error_str)
		cls._logger.info('IQAROS factory reset successful.')

	@classmethod
	def reset_network_manager(cls):
		"""
		Perform NetworkManager factory reset.

		Returns
		-------
		None
		"""
		cls._logger.info('Performing NetworkManager factory reset.')
		try:
			FileManager.copy_tree(
				source=f'{cls._backup_dir}/{SoftwareBackup.NETWORK_MANAGER}',
				destination='/etc/NetworkManager',
			)
			cls._logger.info('NetworkManager factory reset successful.')
		except IOErrorException as io_error:
			cls._logger.error(str(io_error))
			raise io_error

	@classmethod
	def reset_webapp(cls):
		"""
		Perform Webapp factory reset.

		Returns
		-------
		None
		"""
		cls._logger.info('Performing IQRF Gateway Webapp factory reset.')
		try:
			FileManager.remove('/var/lib/iqrf-gateway-webapp/database.db')
			FileManager.remove_tree('/var/cache/iqrf-gateway-webapp/cache/')
			FileManager.remove_tree('/var/cache/iqrf-gateway-webapp/sessions/')
		except IOErrorException as io_error:
			cls._logger.error(str(io_error))
			raise io_error
		WebappConfig.configure_database()
		cls._logger.info('IQRF Gateway Webapp factory reset successful.')
