"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import json
import os.path
import re
from logging import Logger
from typing import Union

from influxdb import InfluxDBClient
from packaging import version

from Setter.Common.Command import Command
from Setter.Common.PasswordGenerator import PasswordGenerator
from Setter.InfluxDbConfig import InfluxDbConfig
from Setter.Logging import Logging
from Setter.MosquittoConfig import MosquittoConfig


class InfluxDbBridgeConfig:
	"""
	IQRF MQTT-InfluxDB bridge config.

	Methods
	-------
	configure() -> None:
		Configure IQRF MQTT-InfluxDB bridge.
	configure_influx() -> None:
		Configure InfluxDB database and user.
	"""

	__config_file = '/etc/iqrf-gateway-influxdb-bridge/config.json'
	__logger: Logger = Logging.get_logger()
	__influx_db = 'iqrf'
	__influx_username = 'iqrf-gateway-influxdb-bridge'
	__influx_password = ''
	__mqtt_username = 'iqrf-gateway-influxdb-bridge'
	__mqtt_password = ''
	__version: version.Version = None

	@classmethod
	def configure(cls, generic_iqaros: bool = False) -> None:
		"""
		Configure IQRF MQTT-InfluxDB bridge.

		Parameters
		----------
		generic_iqaros: bool
			configure influxdb bridge for generic iqaros

		Returns
		-------
		None
		"""
		cls.__logger.info('Configuring IQRF MQTT-InfluxDB bridge.')
		if not os.path.exists(cls.__config_file):
			cls.__logger.warning('IQRF MQTT-InfluxDB bridge configuration file not found.')
			return
		cls.__version = cls.get_version()
		if cls.__version is None:
			return
		cls.__influx_password = PasswordGenerator.generate()
		cls.__mqtt_password = PasswordGenerator.generate()
		cls.configure_influx(generic_iqaros=generic_iqaros)
		cls.configure_mqtt()
		with open(cls.__config_file, 'r+', encoding='UTF-8') as file:
			config = json.load(file)
			config['influx']['user'] = cls.__influx_username
			config['influx']['password'] = cls.__influx_password
			config['influx']['token'] = f'{cls.__influx_username}:{cls.__influx_password}'
			if cls.__version.major == 1:
				config['influx']['bucket'] = cls.__influx_db
			config['mqtt']['host'] = 'localhost'
			config['mqtt']['port'] = 1883
			config['mqtt']['client'] = 'mqtt-influxdb-bridge'
			config['mqtt']['topics'] = ['gateway/+/iqrf/responses']
			config['mqtt']['user'] = cls.__mqtt_username
			config['mqtt']['password'] = cls.__mqtt_password
			file.seek(0)
			json.dump(config, file, indent='\t', separators=(',', ': '), sort_keys=True)
			file.truncate()
			file.close()

	@classmethod
	def configure_influx(cls, generic_iqaros: bool = False) -> None:
		"""
		Configure InfluxDB database and user.

		Parameters
		----------
		generic_iqaros: bool
			configure influxdb for generic iqaros

		Returns
		-------
		None
		"""
		try:
			client: InfluxDBClient = InfluxDbConfig.create_client()
			client.create_database(cls.__influx_db)
			duration = f'{30 if generic_iqaros else 8}d'
			client.alter_retention_policy('autogen', database=cls.__influx_db, duration=duration)
			if not any(user.get('user') == cls.__influx_username for user in client.get_list_users()):
				client.create_user(cls.__influx_username, cls.__influx_password)
			else:
				client.set_user_password(cls.__influx_username, cls.__influx_password)
			client.grant_privilege('all', cls.__influx_db, cls.__influx_username)
			client.close()
		except FileNotFoundError:
			cls.__logger.error('Failed to configure InfluxDB database and user because InfluxDB credentials file not found')

	@classmethod
	def configure_mqtt(cls) -> None:
		"""
		Configure MQTT user.

		Returns
		-------
		None

		"""
		MosquittoConfig.create_user(cls.__mqtt_username, cls.__mqtt_password)

	@classmethod
	def get_version(cls) -> Union[version.Version, None]:
		"""
		Get IQRF MQTT-InfluxDB bridge version.

		Returns
		-------
		Union[Version, None]
			IQRF MQTT-InfluxDB bridge version
		"""
		command: str = 'iqrf-gateway-influxdb-bridge --version'
		result = Command.execute(command)
		if result.returncode != 0:
			cls.__logger.error('Failed to get IQRF MQTT-InfluxDB bridge version: %s.', result.stderr)
			return None
		pattern = r'IQRF Gateway Influx[dD][bB] Bridge v(?P<version>\d+\.\d+\.\d+(-(alpha|beta|rc)~\d+)?)'
		match = re.search(pattern, result.stdout)
		if match is None:
			cls.__logger.error('Failed to get IQRF MQTT-InfluxDB bridge version: %s.', result.stdout)
			return None
		return version.parse(match.group('version').replace('~', '-'))
