"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import os
import json
from logging import Logger
from time import sleep

from influxdb import InfluxDBClient

from Setter.Logging import Logging
from Setter.Common.ServiceManager import ServiceManager


class InfluxDbConfig:
	"""
	InfluxDB config.

	Methods
	-------
	configure() -> None:
		Configure IQRF MQTT-InfluxDB bridge.
	"""

	__credentials_file = '/etc/iqrf-gateway-setter/influxdb-credentials.json'
	__logger: Logger = Logging.get_logger()

	@classmethod
	def configure_admin_password(cls, password: str) -> None:
		"""
		Configure InfluxDB.

		Parameters
		----------
		password: str
			Password for the admin user

		Returns
		-------
		None
		"""
		if not os.path.exists(cls.__credentials_file):
			cls.__logger.error('InfluxDB credentials file not found')
			return
		client: InfluxDBClient = InfluxDbConfig.create_client()
		with open(cls.__credentials_file, 'r+', encoding='UTF-8') as file:
			credentials = json.load(file)
			if credentials['password'] != 'iqrf':
				file.close()
				client.close()
				return
			credentials['password'] = password
			client.set_user_password(credentials['username'], password)
			file.seek(0)
			json.dump(credentials, file, indent='\t', separators=(',', ': '), sort_keys=True)
			file.truncate()
			file.close()
		client.close()

	@classmethod
	def create_client(cls) -> InfluxDBClient:
		"""
		Create a new InfluxDB client instance.

		Raises
		------
		FileNotFoundError
			InfluxDB credentials file not found
		"""
		if not os.path.exists(InfluxDbConfig.__credentials_file):
			raise FileNotFoundError('InfluxDB credentials file not found')
		with open(InfluxDbConfig.__credentials_file, 'r', encoding='UTF-8') as file:
			credentials = json.load(file)
		activating = ServiceManager.is_activating('influxdb')
		while activating:
			cls.__logger.warning('InfluxDB is activating, checking to see if running again in 5 seconds.')
			sleep(5)
			activating = ServiceManager.is_activating('influxdb')
		running = ServiceManager.is_running('influxdb')
		if not running:
			raise RuntimeError('InfluxDB not running, cannot configure.')
		return InfluxDBClient(
			host='localhost',
			port=8086,
			username=credentials['username'],
			password=credentials['password'],
		)
